*
* TDBS Enhanced Text Reading Utility for RIP
* Revision 1.00
* Original ANSI version written by Alan D. Bryant for eSoft, Inc.
*
****************************************************************************
* This program source is included with your TDBS to illustrate techniques. *
* You may use any or all of the source or techniques illustrated in this   *
* program in any fashion you wish.  There are no fees or restrictions      *
* imposed on the use of this code by eSoft, Inc.                           *
****************************************************************************
*
* PURPOSE:  This program can be used as an enhancement to TBBS' internal
* Type 1 menu command.  It is designed to display a text file to the
* screen one page at a time, allow forward and backward movement in the
* file, and download ability on the file.  It also displays a UNIX-like
* "percent complete" measurement.
*
* WARNING:  This program is valid for RIP users only!
*
* POSSIBLE ENHANCEMENTS:  The program as written now leaves the temp
* databases once the program is done.  It could easily be enhanced
* to "clean-up" (delete) the temporary databases when done with them.
* It could also be configured to access multiple files, selectively
* enable/disable downloading, etc.  You could also make it return
* to the program after the download (it currently exits).
*
* make temporary database filename based on unique line number
*
myfile = "TR"+ULINE()+".DBF"

*
* if an old version of that file exists, delete it
*
if file(myfile)
    erase &myfile
endif

*
* create the database in real time
*
create newstruc
use newstruc
append blank
replace field_name with "FPOS", field_type with "N", field_len with 10
use
create &myfile from newstruc
erase newstruc.dbf

*
* use the database we just created
*
use &myfile

*
* look for filename in the opt data; it comes after the "&&"
*
offset = at(chr(38)+chr(38), optdata())
foobar = trim(substr(optdata(), offset + 3))


*
* set initial value for file position
*
pos = 0


*
* open the text file in read mode, and report an error if there was one
*
fopen handle (foobar) 10
if handle = -1
    ? "Error: "+MESSAGE(FERROR())
    ? "File was: ["+foobar+"]"
    wait
    quit
endif


*
* get the size of the file we're reading so we can display the
* percentage done
*
foosize = fsize(foobar)

*
* establish the initial value for a counter
*
counter = 1

*
* do initial screen prep
*
clear
? chr(01)+"|1K|*|c0E|W00|w0004271100|S0107|B0000HR0V|B008GHR99"
? "!|Y06000200|@0302TBBS Enhanced Reader"
? "!|Y00000100|1B00000201EQ0300070F080700020F07000000"
? "!|1U0A8N2Z932B00<>Stop<>S|1U378N5X932800<>Previous<>P"
? "!|1UAV8NDL931W00<>Download<>D"
? "!|1U658NAN932620<>Next<>N"
? "!|c0F|@8I0D"+foobar
? ""
*
* get and store our initial file offset to our temp database
*
fseek handle pos 0
page = 1
append blank
replace FPOS with pos

*
* set colors for the text
*
set color to w+/

*
* start a loop for our file processing
*
do while .t.
    *
    * if we've displayed 32 lines to the screen, it's time
    * for a pause

    if counter = 32
        *
        * reset the counter
        *
        counter = 1

        *
        * get our current file offset, then calculate the percentage done
        *
        fseek handle moof 0
        percent = (moof * 100) / foosize


        *
        * display the prompt line and wait for input
        *
        ? chr(01)+"|S0107|BDR8HHN97|c0E|@E58RPage|@FA8R"+ltrim(str(page))+"|@G78R"+ltrim(str(percent))+"%"
        ? ""
        key = inkey(0)
        sel = upper(chr(key))

        *
        * handle the prompt line selection
        *

        if sel = "S"    && stop
            ? chr(01)+"|*"
            ? ""
            quit
        endif

        if sel = "P"    && previous page
            *
            * decrement our page counter (if possible) then goto that
            * record in our temp database
            *
            if page > 1
                page = page - 1
            endif
            goto page
            *
            * take the offset of that page, and move our pointer in the
            * text file there and then loop
            *
            fseek handle new FPOS 0
            ? chr(01)+"|e"
            ? ""
            loop
        endif

        if sel = "D"    && download
            *
            * download then exit
            *
            if uline()#"00"
              ? chr(01)+"|e"
              ? ""
              dotbbs type 1 optdata foobar+" /D/NBC/NB"
              ? chr(01)+"|*"
              ? ""
              quit
            endif
        endif

        *
        * any other keypress gets us here, so assume next page
        * and handle it as such
        *
        page = page + 1
        if reccount() < page
            fseek handle scratch 0
            append blank
            replace FPOS with scratch
        endif
        ? chr(01)+"|e"
        ? ""
        loop
    endif   && end of "if line count is 33"

    *
    * we're here if the line count is not 33
    *


    *
    * read a line from the file
    *
    flread handle count displ

    *
    * if we actually read any characters, display them and loop
    *
    if count # 0
        ? crtrim(displ)
        counter = counter + 1
        loop
    else
        *
        * we read no bytes, meaning we're at the end of the file
        * handle this case specially, in essence, duplicating what
        * we did above (refer to comments there)
        *
        ? chr(01)+"|S0107|BDR8HHN97|c0E|@E58RPage|@FA8R"+ltrim(str(page))+"|@G78R100%  "
        ? ""
        key = inkey(0)
        sel = upper(chr(key))
        if sel = "D"
            ? chr(01)+"|e"
            ? ""
            dotbbs type 1 optdata foobar+" /D/NBC/NB"
            ? chr(01)+"|*"
            ? ""
            quit
        endif
        if sel = "P"
            if page > 1
                page = page - 1
            endif
            goto page
            fseek handle new FPOS 0
            counter = 1
            ? chr(01)+"|e"
            ? ""
            loop
        endif
        ? chr(01)+"|*"
        ? ""
        quit
    endif
enddo

*
* if we're here, we're done, so close stuff and quit
*
fclose(handle)

* reset RIP screens
? chr(01)+"|*"
? ""

quit
